@@ -1,7 +1,7 @@
 """Define V2 and V3 SimpliSafe systems."""
 from __future__ import annotations
 
-from dataclasses import dataclass
+import dataclasses
 from datetime import datetime
 from enum import Enum
 from functools import wraps
@@ -18,8 +18,6 @@ from simplipy.util.string import convert_to_underscore
 if TYPE_CHECKING:
     from simplipy.api import API
 
-EVENT_SYSTEM_NOTIFICATION = "system_notification"
-
 CONF_DEFAULT = "default"
 CONF_DURESS_PIN = "duress"
 CONF_MASTER_PIN = "master"
@@ -29,7 +27,7 @@ MAX_PIN_LENGTH = 4
 RESERVED_PIN_LABELS = {CONF_DURESS_PIN, CONF_MASTER_PIN}
 
 
-@dataclass(frozen=True)
+@dataclasses.dataclass(frozen=True)
 class SystemNotification:
     """Define a representation of a system notification."""
 
@@ -95,7 +93,7 @@ def guard_from_missing_data(default_value: Any = None) -> Callable:
     return decorator
 
 
-class System:
+class System:  # pylint: disable=too-many-public-methods
     """Define a system.
 
     Note that this class shouldn't be instantiated directly; it will be instantiated as
@@ -212,6 +210,10 @@ class System:
             self._api.subscription_data[self._sid]["location"]["system"]["version"],
         )
 
+    async def _async_clear_notifications(self) -> None:
+        """Clear active notifications."""
+        raise NotImplementedError()
+
     async def _async_set_state(self, value: SystemStates) -> None:
         """Raise if calling this undefined based method."""
         raise NotImplementedError()
@@ -232,6 +234,23 @@ class System:
         """Update subscription data."""
         await self._api.async_update_subscription_data()
 
+    def as_dict(self) -> dict[str, Any]:
+        """Return dictionary version of this device."""
+        return {
+            "address": self.address,
+            "alarm_going_off": self.alarm_going_off,
+            "connection_type": self.connection_type,
+            "notifications": [
+                dataclasses.asdict(notification) for notification in self.notifications
+            ],
+            "serial": self.serial,
+            "state": self.state.value,
+            "system_id": self.system_id,
+            "temperature": self.temperature,
+            "version": self.version,
+            "sensors": [sensor.as_dict() for sensor in self.sensors.values()],
+        }
+
     async def async_clear_notifications(self) -> None:
         """Clear all active notifications.
 
@@ -239,9 +258,7 @@ class System:
         longer visible in the SimpliSafe mobile and web apps.
         """
         if self._notifications:
-            await self._api.request(
-                "delete", f"subscriptions/{self.system_id}/messages"
-            )
+            await self._async_clear_notifications()
             self._notifications = []
 
     def generate_device_objects(self) -> None:
@@ -267,7 +284,7 @@ class System:
         if num_events:
             params["numEvents"] = num_events
 
-        events_resp = await self._api.request(
+        events_resp = await self._api.async_request(
             "get", f"subscriptions/{self.system_id}/events", params=params
         )
 
