@@ -200,7 +200,7 @@ class WebsocketClient:
     """A websocket connection to the SimpliSafe cloud.
 
     Note that this class shouldn't be instantiated directly; it will be instantiated as
-    appropriate via :meth:`simplipy.API.async_from_auth` or
+    appropriate via :meth:`simplipy.API.async_from_credentials` or
     :meth:`simplipy.API.async_from_refresh_token`.
 
     :param api: A :meth:`simplipy.API` object
